package com.giantleap.common.nodebean.node.base.storage.path;

import com.giantleap.common.nodebean.node.base.storage.path.exception.IllegalNameException;




public class NameImp implements NameInterface{
    /** The internalized namespace URI of this qualified name. */
    private final String namespaceURI;

    /** The local part of this qualified name. */
    private final String localName;
    

  
    
    public NameImp(String namespaceURI, String localName) {
        // internalize namespaceURI to improve performance of comparisons.            
        this.namespaceURI = namespaceURI.intern();
        // localName is not internalized in order not to risk huge perm
        // space for large repositories
        this.localName = localName;
    }

    
    //-----------------------------------------------------------< Name >---
    /**
     * @see NameInterface#getLocalName()
     */
    public String getLocalName() {
        return localName;
    }

    /**
     * @see NameInterface#getNamespaceURI()
     */
    public String getNamespaceURI() {
        return namespaceURI;
    }

    //---------------------------------------------------------< Object >---
    /**
     * Returns the string representation of this <code>Name</code> in the
     * following format:
     * <p/>
     * <code><b>{</b>namespaceURI<b>}</b>localName</code>
     *
     * @return the string representation of this <code>Name</code>.
     * @see NameFactory#create(String)
     * @see Object#toString()
     */
    public String toString() {
    	if(this.equals(ROOT_NAME)){
    		return PathInterface.ROOT_PATH_LITERAL;
    	}
    	if(namespaceURI!=null&&!namespaceURI.trim().equals(""))
    	return NameInterface.NAMESPACE_NAME_DELIMITER_LEFT+namespaceURI.trim() + NameInterface.NAMESPACE_NAME_DELIMITER_RIGHT + localName;
    	return localName;
    }
    
    public String getXMLString(){
    	if(this.equals(ROOT_NAME)){
    		return PathInterface.ROOT_PATH_LITERAL;
    	}
    	if(!namespaceURI.equals(NS_DEFAULT_URI)){
    		return namespaceURI+NameInterface.NAMESPACE_NAME_XML_DELIMITER + localName;
    	}else{
    		return localName;
    	}	
    }

    /**
     * Compares two qualified names for equality. Returns <code>true</code>
     * if the given object is a qualified name and has the same namespace URI
     * and local part as this qualified name.
     *
     * @param obj the object to compare this qualified name with
     * @return <code>true</code> if the object is equal to this qualified name,
     *         <code>false</code> otherwise
     * @see Object#equals(Object)
     */
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj instanceof NameImp) {
            NameImp other = (NameImp) obj;
            // we can use == operator for namespaceURI since it is internalized
            return namespaceURI == other.namespaceURI && localName.equals(other.localName);
        }
        // some other Name implementation
        if (obj instanceof NameInterface) {
        	NameInterface other = (NameInterface) obj;
            return namespaceURI.equals(other.getNamespaceURI()) && localName.equals(other.getLocalName());
        }
        return false;
    }

    /**
     * Returns the hash code of this qualified name. The hash code is
     * computed from the namespace URI and local part of the qualified
     * name and memorized for better performance.
     *
     * @return hash code
     * @see Object#hashCode()
     */
    public int hashCode() {
        // Name is immutable, we can store the computed hash code value
        int h = 17;
        h = 37 * h + namespaceURI.hashCode();
        h = 37 * h + localName.hashCode();     
        return h;
    }

    //------------------------------------------------------< Cloneable >---
    /**
     * Creates a clone of this qualified name.
     * Overriden in order to make <code>clone()</code> public.
     *
     * @return a clone of this instance
     * @throws CloneNotSupportedException never thrown
     * @see Object#clone()
     */
    public Object clone() throws CloneNotSupportedException {
        // Name is immutable, no special handling required
        return super.clone();
    }

    //-----------------------------------------------------< Comparable >---
    /**
     * Compares two qualified names.
     *
     * @param o the object to compare this qualified name with
     * @return comparison result
     * @throws ClassCastException if the given object is not a qualified name
     * @see Comparable#compareTo(Object)
     */
    public int compareTo(Object o) {
        if (this == o) {
            return 0;
        }
        NameInterface other = (NameInterface) o;
        if (namespaceURI.equals(other.getNamespaceURI())) {
            return localName.compareTo(other.getLocalName());
        } else {
            return namespaceURI.compareTo(other.getNamespaceURI());
        }
    }

    //---------------------------------------------------< Serializable >---
    /**
     * Creates a new <code>Name</code> instance using the proper constructor
     * during deserialization in order to make sure that internalized strings
     * are used where appropriate.
     */
    private Object readResolve() {
        return new NameImp(namespaceURI, localName);
    }
    
    public static NameInterface createName(String namespaceURI, String localName){
    	// NOTE: an empty localName and/or URI is valid (e.g. the root node name)
    	if (namespaceURI == null) {
            throw new IllegalArgumentException("invalid namespaceURI specified");
        }
        if (localName == null) {
            throw new IllegalArgumentException("invalid localName specified");
        }
        return new NameImp(namespaceURI, localName);
    }

    public static NameInterface createName(String nameString) throws IllegalNameException {        
    	String[] strs= doParse(nameString);
    	return new NameImp(strs[0], strs[1]);
    }
    
    private static String[] doParse(String name) throws IllegalNameException {
        if (name == null || name.length() == 0)throw new IllegalNameException("empty name");        
        name=name.trim();
        if(name.equals("") )throw new IllegalNameException("empty name");
        
        String prefix=null;
        String localName=null;
        if(name.contains(NAMESPACE_NAME_DELIMITER_RIGHT)){
            if (name.charAt(0) != NAMESPACE_NAME_DELIMITER_LEFT_CHAR) {
                throw new IllegalArgumentException("Invalid Name literal:"+name);
            }
            int i = name.indexOf(NAMESPACE_NAME_DELIMITER_RIGHT_CHAR);
            if (i == -1) {
                throw new IllegalArgumentException("Invalid Name literal");
            }
            if (i == name.length() - 1) {
                throw new IllegalArgumentException("Invalid Name literal");
            }
            prefix=name.substring(1, i);
            localName=name.substring(i + 1).trim();
        }else if(name.contains(NAMESPACE_NAME_XML_DELIMITER)){
        	int i =name.indexOf(NAMESPACE_NAME_XML_DELIMITER_CHAR);
        	prefix=name.substring(0, i);
        	localName=name.substring(i+1).trim();
        }else{
            prefix=NS_DEFAULT_URI;
            localName=name;
        }

        return new String[]{prefix, localName};
    }
    
}
