package com.giantleap.common.nodebean.node.base.storage.path;

import com.giantleap.common.nodebean.node.base.storage.path.exception.IllegalNameException;

public class PathElementImp implements PathElement {

    /**
     * Qualified name of the path element.
     */
    private final NameInterface name;

    /**
     * Optional index of the path element. Set to zero, if not
     * explicitly specified, otherwise contains the 1-based index.
     */
    private final int index;
    
    /**
     *NORMAL_TYPE=0
     *ROOT_TYPE = 1;
     *CURRENT_TYPE = 2;
     *PARENT_TYPE = 4;
     * 
     */
    private final int type;
    //private final static Name ROOT_NAME = NAME_FACTORY.create(Name.NS_DEFAULT_URI, "");
    /**
     * Private constructor for creating a path element with the given
     * qualified name and index. Instead of using this constructor directly
     * the factory methods {@link PathFactory#createElement(NameInterface)} and
     * {@link PathFactory#create(NameInterface, int)} should be used.
     *
     * @param name  qualified name
     * @param index index
     */
    public PathElementImp(NameInterface name, int index) {
        this.index = index;
        this.name = name;
        this.type=0;
    }

    private PathElementImp(NameInterface name, int index,int type) {
        this.index = index;
        this.name = name;
        this.type=type;
    }
    
    static PathElementImp createPathElement(NameInterface name, int index,int type) {
    	return new PathElementImp(name,index,type);
    }
    
    public static PathElement createPathElement(NameInterface name) throws IllegalNameException {
        if (name == null) {
            throw new IllegalArgumentException("name must not be null");
        } else if (name.equals(NameInterface.PARENT_NAME)) {
            return PARENT_ELEMENT;
        } else if (name.equals(NameInterface.CURRENT_NAME)) {
            return CURRENT_ELEMENT;
        } else if (name.equals(NameInterface.ROOT_NAME)) {
            return ROOT_ELEMENT;
        } else {
            return new PathElementImp(name, PathElement.INDEX_UNDEFINED);
        }
    }
    public static PathElement createPathElement(NameInterface name, int index) throws IllegalNameException {
        if (index < PathElement.INDEX_UNDEFINED) {
            throw new IllegalNameException("The index may not be negative.");
        } else if (name == null) {
            throw new IllegalNameException("The name must not be null");
        } else if (name.equals(NameInterface.PARENT_NAME)|| name.equals(NameInterface.CURRENT_NAME)|| name.equals(NameInterface.ROOT_NAME)) {
            throw new IllegalNameException("Special path elements (root, '.' and '..') can not have an explicit index.");
        } else {
            return new PathElementImp(name, index);
        }
    }
    
    public static PathElement createPathElement(String elementString)throws IllegalNameException {
    	return doParse(elementString);
    }
    
    private static PathElement doParse(String elementString)throws IllegalNameException{
        if (elementString == null||elementString.trim().equals("")) {
            throw new IllegalArgumentException("null PathElement literal");
        }
        int pos = elementString.indexOf(PathElement.INDEX_DELIMITER_LEFT_CHAR);
        NameInterface name=null;

  
            if (pos == -1) {
                name = NameImp.createName(elementString);
                return new PathElementImp(name, PathElement.INDEX_DEFAULT);
            }
            name = NameImp.createName(elementString.substring(0, pos));          

        int pos1 = elementString.indexOf(PathElement.INDEX_DELIMITER_RIGHT_CHAR);
        if (pos1 == -1) {
            throw new IllegalArgumentException("invalid PathElement literal: " + elementString + " (missing ']')");
        }
     
            int index = Integer.valueOf(elementString.substring(pos + 1, pos1)).intValue();
            //System.out.println("index:" +index);
            if (index < PathElement.INDEX_DEFAULT) {
            	index=PathElement.INDEX_DEFAULT;
                //throw new IllegalArgumentException("invalid PathElement literal: " + elementString + " (index is 1-based)");
            }
            return new PathElementImp(name, index);        
    }
    
    
    /**
     * @see Path.Element#getName()
     */
    public NameInterface getName() {
        return name;
    }

    /**
     * @see Path.Element#getIndex()
     */
    public int getIndex() {
        return index;
    }

    /**
     * @see Path.Element#getNormalizedIndex()
     */
    public int getNormalizedIndex() {
        if (index == PathElement.INDEX_UNDEFINED) {
            return PathElement.INDEX_DEFAULT;
        } else {
            return index;
        }
    }

    /**
     * @return always returns false.
     * @see Path.Element#denotesRoot()
     */
    public boolean denotesRoot() {
        return type==1;
    }

    /**
     * @return always returns false.
     * @see Path.Element#denotesParent()
     */
    public boolean denotesParent() {
    	return type==4;
    }

    /**
     * @return always returns false.
     * @see Path.Element#denotesCurrent()
     */
    public boolean denotesCurrent() {
    	return type==2;
    }

    /**
     * @return always returns true.
     * @see Path.Element#denotesName()
     */
    public boolean denotesRegularName() {
        return true;
    }

    /**
     * @see Path.Element#getString()
     */
    
    public String getString(){
    	return toString();
    }
    
    public String getXMLString(){
        StringBuffer sb = new StringBuffer();
        // name
        sb.append(name.getXMLString());
        // index
        if (index > PathElement.INDEX_DEFAULT) {
            sb.append(PathElement.INDEX_DELIMITER_LEFT_CHAR);
            sb.append(index);
            sb.append(PathElement.INDEX_DELIMITER_RIGHT_CHAR);
        }
        return sb.toString();
    }

    /**
     * Returns a string representation of this path element. Note that
     * the path element name is expressed using the <code>{uri}name</code>
     * syntax.
     *
     * @return string representation of the path element
     * @see Object#toString()
     */
    public String toString() {
        StringBuffer sb = new StringBuffer();
        // name
        sb.append(name.toString());
        // index
        if (index > PathElement.INDEX_DEFAULT) {
            sb.append(PathElement.INDEX_DELIMITER_LEFT_CHAR);
            sb.append(index);
            sb.append(PathElement.INDEX_DELIMITER_RIGHT_CHAR);
        }
        return sb.toString();
    }

    
    
    /**
     * Computes a hash code for this path element.
     *
     * @return hash code
     * @see Object#hashCode()
     */
    public int hashCode() {
        int h = 17;
        h = 37 * h + getNormalizedIndex();
        h = 37 * h + name.hashCode();
        return h;
    }

    /**
     * Check for path element equality. Returns true if the given
     * object is a PathElement and contains the same name and index
     * as this one.
     *
     * @param obj the object to compare with
     * @return <code>true</code> if the path elements are equal
     * @see Object#equals(Object)
     */
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj instanceof PathElement) {
        	PathElement other = (PathElement) obj;
            return name.equals(other.getName())
                    && getNormalizedIndex() == other.getNormalizedIndex();
        }
        return false;
    }
}