package com.giantleap.common.nodebean.node.base.storage.value;

import java.util.Calendar;
import java.util.Date;

import com.giantleap.common.util.dataformat.ISO8601;

/**
 * A <code>DateValue</code> provides an implementation of the <code>Value</code>
 * interface representing a date value.
 */
public class DateValue extends BaseValue {

	public static final int TYPE = PropertyType.DATE;

	private final Calendar date;

	/**
	 * Constructs a <code>DateValue</code> object representing a date.
	 * 
	 * @param date
	 *            the date this <code>DateValue</code> should represent
	 */
	public DateValue(Calendar date) {
		super(TYPE);
		this.date = date;
	}

	/**
	 * Returns a new <code>DateValue</code> initialized to the value represented
	 * by the specified <code>String</code>.
	 * <p/>
	 * The specified <code>String</code> must be a ISO8601-compliant date/time
	 * string.
	 * 
	 * @param s
	 *            the string to be parsed.
	 * @return a newly constructed <code>DateValue</code> representing the the
	 *         specified value.
	 * @throws javax.jcr.ValueFormatException
	 *             If the <code>String</code> is not a valid ISO8601-compliant
	 *             date/time string.
	 * @see ISO8601
	 */
	public static DateValue valueOf(String s) throws IllegalValueException {
		// System.out.println(s.equals("1997-07-16T19:20:30.451+01:00"));
		Calendar cal = ISO8601.parse(s);
		if (cal != null) {
			return new DateValue(cal);
		} else {
			throw new IllegalValueException("not a valid date format: " + s);
		}
	}

	public static String formatCalendar(Calendar cal) {
		return ISO8601.format(cal);
	}

	public static String formatDate(Date date) {
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(date);
		return formatCalendar(calendar);
	}

	/**
	 * Indicates whether some other object is "equal to" this one.
	 * <p/>
	 * The result is <code>true</code> if and only if the argument is not
	 * <code>null</code> and is a <code>DateValue</code> object that represents
	 * the same value as this object.
	 * <p>
	 * The value comparison is performed using the ISO 8601 string
	 * representation of the dates, since the native Calendar.equals() method
	 * may produce false negatives (see JSR-598).
	 * <p>
	 * Note that the comparison still returns false when comparing the same time
	 * in different time zones, but that seems to be the intent of JSR 170.
	 * Compare the Value.getDate().getTime() values if you need an exact time
	 * comparison in UTC.
	 * 
	 * @param obj
	 *            the reference object with which to compare.
	 * @return <code>true</code> if this object is the same as the obj argument;
	 *         <code>false</code> otherwise.
	 */
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj instanceof DateValue) {
			DateValue other = (DateValue) obj;
			if (date == other.date) {
				return true;
			} else if (date != null && other.date != null) {
				return ISO8601.format(date).equals(ISO8601.format(other.date));
			}
		}
		return false;
	}

	/**
	 * Returns zero to satisfy the Object equals/hashCode contract. This class
	 * is mutable and not meant to be used as a hash key.
	 * 
	 * @return always zero
	 * @see Object#hashCode()
	 */
	public int hashCode() {
		return 0;
	}

	// ------------------------------------------------------------< BaseValue >
	/**
	 * {@inheritDoc}
	 */
	protected String getInternalString() throws IllegalValueException {
		if (date != null) {
			return ISO8601.format(date);
		} else {
			throw new IllegalValueException("empty value");
		}
	}

	// ----------------------------------------------------------------< Value >
	/**
	 * {@inheritDoc}
	 */
	public Calendar getDate() throws IllegalValueException{
		setValueConsumed();

		if (date != null) {
			return date;
		} else {
			throw new IllegalValueException("empty value");
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public long getLong() throws IllegalValueException {
		setValueConsumed();

		if (date != null) {
			return date.getTimeInMillis();
		} else {
			throw new IllegalValueException("empty value");
		}
	}

	public int getInt() throws IllegalValueException{
		setValueConsumed();

		throw new IllegalValueException("conversion to long failed: inconvertible types");
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean getBoolean() throws IllegalValueException {
		setValueConsumed();

		if (date != null) {
			throw new IllegalValueException("cannot convert date to boolean");
		} else {
			throw new IllegalValueException("empty value");
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public double getDouble() throws IllegalValueException {
		setValueConsumed();

		if (date != null) {
			long ms = date.getTimeInMillis();
			if (ms <= Double.MAX_VALUE) {
				return ms;
			}
			throw new IllegalValueException("conversion from date to double failed: inconvertible types");
		} else {
			throw new IllegalValueException("empty value");
		}
	}
}
