package com.giantleap.common.nodebean.node.base.storage.value;

import java.util.Calendar;
import java.util.Date;

public class FloatValue extends BaseValue {

	public static final int TYPE = PropertyType.FLOAT;

	private final Float dblNumber;

	/**
	 * Constructs a <code>DoubleValue</code> object representing a double.
	 * 
	 * @param dblNumber
	 *            the double this <code>DoubleValue</code> should represent
	 */
	public FloatValue(Double dblNumber) {
		super(TYPE);
		this.dblNumber = dblNumber.floatValue();
	}
	
	public FloatValue(Float dblNumber) {
		super(TYPE);
		this.dblNumber = dblNumber;
	}

	/**
	 * Constructs a <code>DoubleValue</code> object representing a double.
	 * 
	 * @param dbl
	 *            the double this <code>DoubleValue</code> should represent
	 */
	public FloatValue(double dbl) {
		super(TYPE);
		this.dblNumber = new Float(dbl);
	}
	public FloatValue(float dbl) {
		super(TYPE);
		this.dblNumber = new Float(dbl);
	}
	/**
	 * Returns a new <code>DoubleValue</code> initialized to the value
	 * represented by the specified <code>String</code>.
	 * 
	 * @param s
	 *            the string to be parsed.
	 * @return a newly constructed <code>DoubleValue</code> representing the the
	 *         specified value.
	 * @throws javax.jcr.ValueFormatException
	 *             If the <code>String</code> does not contain a parsable
	 *             <code>double</code>.
	 */
	public static FloatValue valueOf(String s) throws IllegalValueException {
		try {
			return new FloatValue(Float.parseFloat(s));
		} catch (NumberFormatException e) {
			throw new IllegalValueException("invalid format", e);
		}
	}

	/**
	 * Indicates whether some other object is "equal to" this one.
	 * <p/>
	 * The result is <code>true</code> if and only if the argument is not
	 * <code>null</code> and is a <code>DoubleValue</code> object that
	 * represents the same value as this object.
	 * 
	 * @param obj
	 *            the reference object with which to compare.
	 * @return <code>true</code> if this object is the same as the obj argument;
	 *         <code>false</code> otherwise.
	 */
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj instanceof FloatValue) {
			FloatValue other = (FloatValue) obj;
			if (dblNumber == other.dblNumber) {
				return true;
			} else if (dblNumber != null && other.dblNumber != null) {
				return dblNumber.equals(other.dblNumber);
			}
		}
		return false;
	}

	/**
	 * Returns zero to satisfy the Object equals/hashCode contract. This class
	 * is mutable and not meant to be used as a hash key.
	 * 
	 * @return always zero
	 * @see Object#hashCode()
	 */
	public int hashCode() {
		return 0;
	}

	// ------------------------------------------------------------< BaseValue >
	/**
	 * {@inheritDoc}
	 */
	protected String getInternalString() throws IllegalValueException {
		if (dblNumber != null) {
			return dblNumber.toString();
		} else {
			throw new IllegalValueException("empty value");
		}
	}

	// ----------------------------------------------------------------< Value >
	/**
	 * {@inheritDoc}
	 */
	public Calendar getDate() throws IllegalValueException {
		setValueConsumed();

		if (dblNumber != null) {
			// loosing timezone information...
			Calendar cal = Calendar.getInstance();
			cal.setTime(new Date(dblNumber.longValue()));
			return cal;
		} else {
			throw new IllegalValueException("empty value");
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public long getLong() throws IllegalValueException {
		setValueConsumed();

		if (dblNumber != null) {
			return dblNumber.longValue();
		} else {
			throw new IllegalValueException("empty value");
		}
	}

	public int getInt() throws IllegalValueException {
		setValueConsumed();

		if (dblNumber != null) {
			return dblNumber.intValue();
		} else {
			throw new IllegalValueException("empty value");
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean getBoolean() throws IllegalValueException {
		setValueConsumed();

		throw new IllegalValueException("conversion to boolean failed: inconvertible types");
	}

	/**
	 * {@inheritDoc}
	 */
	public double getDouble() throws IllegalValueException {
		setValueConsumed();

		if (dblNumber != null) {
			return dblNumber.doubleValue();
		} else {
			throw new IllegalValueException("empty value");
		}
	}
}
