package com.giantleap.common.nodebean.node.base.storage.value;

import java.util.Calendar;
import java.util.Date;

/**
 * A <code>LongValue</code> provides an implementation of the <code>Value</code>
 * interface representing a long value.
 */
public class LongValue extends BaseValue {

	public static final int TYPE = PropertyType.LONG;

	private final Long lNumber;

	/**
	 * Constructs a <code>LongValue</code> object representing a long.
	 * 
	 * @param lNumber
	 *            the long this <code>LongValue</code> should represent
	 */
	public LongValue(Long lNumber) {
		super(TYPE);
		this.lNumber = lNumber;
	}

	/**
	 * Constructs a <code>LongValue</code> object representing a long.
	 * 
	 * @param l
	 *            the long this <code>LongValue</code> should represent
	 */
	public LongValue(long l) {
		super(TYPE);
		this.lNumber = new Long(l);
	}

	/**
	 * Returns a new <code>LongValue</code> initialized to the value represented
	 * by the specified <code>String</code>.
	 * 
	 * @param s
	 *            the string to be parsed.
	 * @return a newly constructed <code>LongValue</code> representing the the
	 *         specified value.
	 * @throws javax.jcr.ValueFormatException
	 *             If the <code>String</code> does not contain a parsable
	 *             <code>long</code>.
	 */
	public static LongValue valueOf(String s) throws IllegalValueException {
		try {
			return new LongValue(Long.parseLong(s));
		} catch (NumberFormatException e) {
			throw new IllegalValueException("invalid format", e);
		}
	}

	/**
	 * Indicates whether some other object is "equal to" this one.
	 * <p/>
	 * The result is <code>true</code> if and only if the argument is not
	 * <code>null</code> and is a <code>LongValue</code> object that represents
	 * the same value as this object.
	 * 
	 * @param obj
	 *            the reference object with which to compare.
	 * @return <code>true</code> if this object is the same as the obj argument;
	 *         <code>false</code> otherwise.
	 */
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj instanceof LongValue) {
			LongValue other = (LongValue) obj;
			if (lNumber == other.lNumber) {
				return true;
			} else if (lNumber != null && other.lNumber != null) {
				return lNumber.equals(other.lNumber);
			}
		}
		return false;
	}

	/**
	 * Returns zero to satisfy the Object equals/hashCode contract. This class
	 * is mutable and not meant to be used as a hash key.
	 * 
	 * @return always zero
	 * @see Object#hashCode()
	 */
	public int hashCode() {
		return 0;
	}

	// ------------------------------------------------------------< BaseValue >
	/**
	 * {@inheritDoc}
	 */
	protected String getInternalString() throws IllegalValueException {
		if (lNumber != null) {
			return lNumber.toString();
		} else {
			throw new IllegalValueException("empty value");
		}
	}

	// ----------------------------------------------------------------< Value >
	/**
	 * {@inheritDoc}
	 */
	public Calendar getDate() throws IllegalValueException {
		setValueConsumed();

		if (lNumber != null) {
			// loosing timezone information...
			Calendar cal = Calendar.getInstance();
			cal.setTime(new Date(lNumber.longValue()));
			return cal;
		} else {
			throw new IllegalValueException("empty value");
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public long getLong() throws IllegalValueException {
		setValueConsumed();

		if (lNumber != null) {
			return lNumber.longValue();
		} else {
			throw new IllegalValueException("empty value");
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean getBoolean() throws IllegalValueException {
		setValueConsumed();

		throw new IllegalValueException("conversion to boolean failed: inconvertible types");
	}

	/**
	 * {@inheritDoc}
	 */
	public double getDouble() throws IllegalValueException {
		setValueConsumed();

		if (lNumber != null) {
			return lNumber.doubleValue();
		} else {
			throw new IllegalValueException("empty value");
		}
	}

	public int getInt() throws IllegalValueException {
		setValueConsumed();
		return lNumber.intValue();
	}

}
