package com.giantleap.common.nodebean.node.base.storage.value;

import java.util.Calendar;

import com.giantleap.common.nodebean.node.base.storage.path.PathImp;
import com.giantleap.common.nodebean.node.base.storage.path.PathInterface;
import com.giantleap.common.nodebean.node.base.storage.path.exception.IllegalNameException;

/**
 * A <code>PathValue</code> provides an implementation of the <code>Value</code>
 * interface representing a <code>PATH</code> value (an absolute or relative
 * workspace path).
 */
public class PathValue extends BaseValue {

	public static final int TYPE = PropertyType.PATH;

	private final String path;

	/**
	 * Returns a new <code>PathValue</code> initialized to the value represented
	 * by the specified <code>String</code>.
	 * <p/>
	 * The specified <code>String</code> must be a valid absolute or relative
	 * path.
	 * 
	 * @param s
	 *            the string to be parsed.
	 * @return a newly constructed <code>PathValue</code> representing the the
	 *         specified value.
	 * @throws javax.jcr.ValueFormatException
	 *             If the <code>String</code> is not a valid absolute or
	 *             relative path.
	 */
	public static PathValue valueOf(String s) throws IllegalValueException {
		if (s != null) {
			PathInterface p = null;
			try {
				p = PathImp.createPath(s);
			} catch (IllegalNameException e) {
				throw new IllegalValueException(e.getMessage());
			}

			return new PathValue(p.toString());
		} else {
			throw new IllegalValueException("not a valid path format");
		}
	}

	/**
	 * Protected constructor creating a <code>PathValue</code> object without
	 * validating the path.
	 * 
	 * @param path
	 *            the path this <code>PathValue</code> should represent
	 * @see #valueOf
	 */
	protected PathValue(String path) {
		super(TYPE);
		this.path = path;
	}

	/**
	 * Indicates whether some other object is "equal to" this one.
	 * <p/>
	 * The result is <code>true</code> if and only if the argument is not
	 * <code>null</code> and is a <code>PathValue</code> object that represents
	 * the same value as this object.
	 * 
	 * @param obj
	 *            the reference object with which to compare.
	 * @return <code>true</code> if this object is the same as the obj argument;
	 *         <code>false</code> otherwise.
	 */
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj instanceof PathValue) {
			PathValue other = (PathValue) obj;
			if (path == other.path) {
				return true;
			} else if (path != null && other.path != null) {
				return path.equals(other.path);
			}
		}
		return false;
	}

	/**
	 * Returns zero to satisfy the Object equals/hashCode contract. This class
	 * is mutable and not meant to be used as a hash key.
	 * 
	 * @return always zero
	 * @see Object#hashCode()
	 */
	public int hashCode() {
		return 0;
	}

	// ------------------------------------------------------------< BaseValue >
	/**
	 * {@inheritDoc}
	 */
	protected String getInternalString() throws IllegalValueException {
		if (path != null) {
			return path;
		} else {
			throw new IllegalValueException("empty value");
		}
	}

	// ----------------------------------------------------------------< Value >
	/**
	 * {@inheritDoc}
	 */
	public Calendar getDate() throws IllegalValueException {
		setValueConsumed();

		throw new IllegalValueException("conversion to date failed: inconvertible types");
	}

	/**
	 * {@inheritDoc}
	 */
	public long getLong() throws IllegalValueException {
		setValueConsumed();

		throw new IllegalValueException("conversion to long failed: inconvertible types");
	}

	public int getInt() throws IllegalValueException {
		setValueConsumed();

		throw new IllegalValueException("conversion to long failed: inconvertible types");
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean getBoolean() throws IllegalValueException {
		setValueConsumed();

		throw new IllegalValueException("conversion to boolean failed: inconvertible types");
	}

	/**
	 * {@inheritDoc}
	 */
	public double getDouble() throws IllegalValueException {
		setValueConsumed();

		throw new IllegalValueException("conversion to double failed: inconvertible types");
	}
}
