package com.giantleap.common.nodebean.node.base.storage.value;

import java.util.HashMap;
import java.util.Map;


/**
 * The property types supported by the JCR standard.
 * <p/>
 * <p>This interface defines following property types:
 * <ul>
 * <li><code>STRING</code>
 * <li><code>BINARY</code>
 * <li><code>LONG</code>
 * <li><code>DOUBLE</code>
 * <li><code>DATE</code>
 * <li><code>BOOLEAN</code>
 * <li><code>NAME</code>
 * <li><code>PATH</code>
 * <li><code>REFERENCE</code>
 * </ul>
 */
public final class PropertyType {
    /*
     * The supported property types.
     */
    public static final int STRING = 1;
    public static final int BINARY = 2;
    public static final int LONG = 3;
    public static final int DOUBLE = 4;
    public static final int DATE = 5;
    public static final int BOOLEAN = 6;
    public static final int NAME = 7;
    public static final int PATH = 8;
    public static final int REFERENCE = 9;
    public static final int HTTPLINK=10;
    public static final int INTEGER = 11;
    
    public static final int DATETIME = 12;
    public static final int FLOAT = 13;
    public static final int ARRAYLIST_STRING =  0x1001;
    public static final int LINKEDLIST_STRING = 0x1101;
    public static final int SET_STRING =  0x1111;
    public static final int MAP_STRING =  0x1100;
    /*
     * Undefined type.
     */
    public static final int UNDEFINED = 0;

    /*
     * The names of the supported property types,
     * as used in serialization.
     */
    public static final String TYPENAME_STRING = "String";
    public static final String TYPENAME_BINARY = "Binary";
    public static final String TYPENAME_LONG = "Long";
    public static final String TYPENAME_DOUBLE = "Double";
    public static final String TYPENAME_FLOAT = "Float";
    public static final String TYPENAME_DATE = "Date";
    public static final String TYPENAME_DATETIME = "DateTime";
    public static final String TYPENAME_BOOLEAN = "Boolean";
    public static final String TYPENAME_NAME = "Name";
    public static final String TYPENAME_PATH = "Path";
    public static final String TYPENAME_REFERENCE = "Reference";
    public static final String TYPENAME_HTTPLINK = "HTTPLink";
    public static final String TYPENAME_INTEGER = "Integer";
    
    public static final String TYPENAME_ARRAYLIST_STRING  = "ARRAYLIST_STRING";
    public static final String TYPENAME_LINKEDLIST_STRING = "LINKEDLIST_STRING";
    public static final String TYPENAME_SET_STRING        = "SET_STRING";
    public static final String TYPENAME_MAP_STRING        = "MAP_STRING";
    /*
     * String representation of <i>undefined</i> type.
     */
    public static final String TYPENAME_UNDEFINED = "undefined";

    
	public static final HashMap<String,Integer> TYPE_CONVERSION=new HashMap<String,Integer>();
	static{
		TYPE_CONVERSION.put("int", PropertyType.INTEGER);
		TYPE_CONVERSION.put("class java.lang.String",PropertyType.STRING);
		TYPE_CONVERSION.put("class java.util.Calendar", PropertyType.DATE);
		TYPE_CONVERSION.put("class java.util.Date", PropertyType.DATETIME);
		TYPE_CONVERSION.put("long", PropertyType.LONG);
		TYPE_CONVERSION.put("double", PropertyType.DOUBLE);
		TYPE_CONVERSION.put("float", PropertyType.FLOAT);
		TYPE_CONVERSION.put("boolean", PropertyType.BOOLEAN);
		
		TYPE_CONVERSION.put("class java.util.LinkedList", PropertyType.LINKEDLIST_STRING);
		TYPE_CONVERSION.put("class java.util.ArrayList",  PropertyType.ARRAYLIST_STRING);
		TYPE_CONVERSION.put("interface java.util.List",  PropertyType.ARRAYLIST_STRING);
		TYPE_CONVERSION.put("interface java.util.Set",  PropertyType.SET_STRING);
		
		TYPE_CONVERSION.put("interface java.util.Map",  PropertyType.MAP_STRING);
		TYPE_CONVERSION.put("class java.util.HashMap",  PropertyType.MAP_STRING);
	}
    
    public static int valueTypeFromJavaClass(String javaClass){
    	Integer type=TYPE_CONVERSION.get(javaClass);
    	if(type!=null){
    		return type.intValue();
    	}else{
    		return PropertyType.STRING;
    	}
    }
    
    
    public static void main(String[] args)throws Exception{
    	//Value value=ValueFactoryImp.createValue("afsda", TYPENAME_STRING);
    	//System.out.println(value.getClass().toString());
    	Map map = new HashMap();
    	System.out.println(map.getClass().toString());
    	System.out.println(valueTypeFromJavaClass(map.getClass().toString()));
    }
    
    /**
     * Returns the name of the specified <code>type</code>,
     * as used in serialization.
     *
     * @param type the property type
     * @return the name of the specified <code>type</code>
     * @throws IllegalArgumentException if <code>type</code>
     *                                  is not a valid property type.
     */
    public static String nameFromValue(int type) {
	switch (type) {
	    case STRING:
		return TYPENAME_STRING;
	    case BINARY:
		return TYPENAME_BINARY;
	    case BOOLEAN:
		return TYPENAME_BOOLEAN;
	    case LONG:
		return TYPENAME_LONG;
	    case DOUBLE:
		return TYPENAME_DOUBLE;
	    case FLOAT:
		return TYPENAME_FLOAT;
	    case DATE:
		return TYPENAME_DATE;
	    case DATETIME:
		return TYPENAME_DATETIME;
	    case NAME:
		return TYPENAME_NAME;
	    case PATH:
		return TYPENAME_PATH;
	    case REFERENCE:
		return TYPENAME_REFERENCE;
	    case INTEGER:
		return TYPENAME_INTEGER;
	    case HTTPLINK:
		return TYPENAME_HTTPLINK;
		
	    case ARRAYLIST_STRING:  return TYPENAME_ARRAYLIST_STRING;
	    case LINKEDLIST_STRING: return TYPENAME_LINKEDLIST_STRING;
	    case SET_STRING:        return TYPENAME_SET_STRING;
	    case MAP_STRING:        return TYPENAME_MAP_STRING;
	    case UNDEFINED:
		return TYPENAME_UNDEFINED;
	    default:
		throw new IllegalArgumentException("unknown type: " + type);
	}
    }

    /**
     * Returns the numeric constant value of the type with the specified name.
     *
     * @param name the name of the property type
     * @return the numeric constant value
     * @throws IllegalArgumentException if <code>name</code>
     *                                  is not a valid property type name.
     */
    public static int valueFromName(String name) {
	if (name.equals(TYPENAME_STRING)) {
	    return STRING;
	} else if (name.equals(TYPENAME_BINARY)) {
	    return BINARY;
	} else if (name.equals(TYPENAME_BOOLEAN)) {
	    return BOOLEAN;
	} else if (name.equals(TYPENAME_LONG)) {
	    return LONG;
	} else if (name.equals(TYPENAME_DOUBLE)) {
	    return DOUBLE;
	} else if (name.equals(TYPENAME_FLOAT)) {
	    return FLOAT;
	} else if (name.equals(TYPENAME_DATE)) {
	    return DATE;
	} else if (name.equals(TYPENAME_DATETIME)) {
	    return DATETIME; 
	}else if (name.equals(TYPENAME_NAME)) {
	    return NAME;
	} else if (name.equals(TYPENAME_PATH)) {
	    return PATH;
	} else if (name.equals(TYPENAME_REFERENCE)) {
	    return REFERENCE;
	} else if (name.equals(TYPENAME_HTTPLINK)) {
	    return HTTPLINK;
	} else if (name.equals(TYPENAME_INTEGER)) {
	    return INTEGER;
	}else if (name.equals(TYPENAME_ARRAYLIST_STRING)) {
	    return ARRAYLIST_STRING;
	}else if (name.equals(TYPENAME_LINKEDLIST_STRING)) {
	    return LINKEDLIST_STRING;
	}else if (name.equals(TYPENAME_SET_STRING)) {
	    return SET_STRING;
	}else if (name.equals(TYPENAME_MAP_STRING)) {
	    return MAP_STRING;
	}else if (name.equals(TYPENAME_UNDEFINED)) {
	    return UNDEFINED;
	} else {
	    throw new IllegalArgumentException("unknown type: " + name);
	}
    }

    /**
     * private constructor to prevent instantiation
     */
    private PropertyType() {
    }
    

    
    
    
}

