package com.giantleap.common.nodebean.node.base.storage.value;

import java.util.Calendar;

import com.giantleap.common.nodebean.node.base.storage.id.UUID;
import com.giantleap.common.nodebean.node.base.storage.node.Node;

/**
 * A <code>ReferenceValue</code> provides an implementation of the
 * <code>Value</code> interface representing a <code>REFERENCE</code> value (a
 * UUID of an existing node).
 */
public class ReferenceValue extends BaseValue {

	public static final int TYPE = PropertyType.REFERENCE;

	private final String uuid;

	/**
	 * Constructs a <code>ReferenceValue</code> object representing the UUID of
	 * an existing node.
	 * 
	 * @param target
	 *            the node to be referenced
	 * @throws IllegalArgumentException
	 *             If <code>target</code> is nonreferenceable.
	 * @throws javax.jcr.RepositoryException
	 *             If another error occurs.
	 */
	public ReferenceValue(Node target) throws IllegalValueException {
		super(TYPE);
		/*
		 * try {
		 */
		this.uuid = target.getId().toString();
		/*
		 * } catch (UnsupportedRepositoryOperationException ure) { throw new
		 * IllegalArgumentException("target is nonreferenceable."); }
		 */
	}

	/**
	 * Returns a new <code>ReferenceValue</code> initialized to the value
	 * represented by the specified <code>String</code>.
	 * <p/>
	 * The specified <code>String</code> must denote the UUID of an existing
	 * node.
	 * 
	 * @param s
	 *            the string to be parsed.
	 * @return a newly constructed <code>ReferenceValue</code> representing the
	 *         the specified value.
	 * @throws javax.jcr.ValueFormatException
	 *             If the <code>String</code> is not a valid not a valid UUID
	 *             format.
	 */
	public static ReferenceValue valueOf(String s) throws IllegalValueException {
		if (s != null) {
			try {
				UUID.fromString(s);
			} catch (IllegalArgumentException iae) {
				throw new IllegalValueException("not a valid UUID format");
			}
			return new ReferenceValue(s);
		} else {
			throw new IllegalValueException("not a valid UUID format");
		}
	}

	/**
	 * Protected constructor creating a <code>ReferenceValue</code> object
	 * without validating the UUID format.
	 * 
	 * @param uuid
	 *            the UUID of the node to be referenced
	 * @see #valueOf
	 */
	protected ReferenceValue(String uuid) {
		super(TYPE);
		this.uuid = uuid;
	}

	/**
	 * Indicates whether some other object is "equal to" this one.
	 * <p/>
	 * The result is <code>true</code> if and only if the argument is not
	 * <code>null</code> and is a <code>ReferenceValue</code> object that
	 * represents the same value as this object.
	 * 
	 * @param obj
	 *            the reference object with which to compare.
	 * @return <code>true</code> if this object is the same as the obj argument;
	 *         <code>false</code> otherwise.
	 */
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj instanceof ReferenceValue) {
			ReferenceValue other = (ReferenceValue) obj;
			if (uuid == other.uuid) {
				return true;
			} else if (uuid != null && other.uuid != null) {
				return uuid.equals(other.uuid);
			}
		}
		return false;
	}

	/**
	 * Returns zero to satisfy the Object equals/hashCode contract. This class
	 * is mutable and not meant to be used as a hash key.
	 * 
	 * @return always zero
	 * @see Object#hashCode()
	 */
	public int hashCode() {
		return 0;
	}

	// ------------------------------------------------------------< BaseValue >
	/**
	 * {@inheritDoc}
	 */
	protected String getInternalString() throws IllegalValueException {
		if (uuid != null) {
			return uuid;
		} else {
			throw new IllegalValueException("empty value");
		}
	}

	// ----------------------------------------------------------------< Value >
	/**
	 * {@inheritDoc}
	 */
	public Calendar getDate() throws IllegalValueException {
		setValueConsumed();

		throw new IllegalValueException("conversion to date failed: inconvertible types");
	}

	/**
	 * {@inheritDoc}
	 */
	public long getLong() throws IllegalValueException {
		setValueConsumed();

		throw new IllegalValueException("conversion to long failed: inconvertible types");
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean getBoolean() throws IllegalValueException {
		setValueConsumed();

		throw new IllegalValueException("conversion to boolean failed: inconvertible types");
	}

	/**
	 * {@inheritDoc}
	 */
	public double getDouble() throws IllegalValueException {
		setValueConsumed();

		throw new IllegalValueException("conversion to double failed: inconvertible types");
	}
}
