package com.giantleap.common.nodebean.node.base.storage.value;

import java.io.InputStream;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Set;

import com.giantleap.common.exception.GLException;
import com.giantleap.common.nodebean.node.base.storage.node.Node;
import com.giantleap.common.util.dataformat.CalendarFormat;
import com.giantleap.common.util.dataformat.ISO8601;

/**
 * This class implements the <code>ValueFactory</code> interface.
 * 
 * @see javax.jcr.Session#getValueFactory()
 */
public class ValueFactoryImp implements ValueFactory {

	private static final ValueFactory valueFactory = new ValueFactoryImp();

	/**
	 * Constructs a <code>ValueFactory</code> object.
	 */
	protected ValueFactoryImp() {
	}

	// --------------------------------------------------------------------------
	/**
     *
     */
	public static ValueFactory getInstance() {
		return valueFactory;
	}

	// ---------------------------------------------------------< ValueFactory >
	/**
	 * {@inheritDoc}
	 */
	public Value createValue(boolean value) {
		return new BooleanValue(value);
	}

	/**
	 * {@inheritDoc}
	 */
	public Value createValue(Calendar value) {
		return new DateValue(value);
	}

	/**
	 * {@inheritDoc}
	 */
	public Value createValue(double value) {
		return new DoubleValue(value);
	}

	/**
	 * {@inheritDoc}
	 */
	public Value createValue(InputStream value) {
		return new BinaryValue(value);
	}

	/**
	 * {@inheritDoc}
	 */
	public Value createValue(long value) {
		return new LongValue(value);
	}

	/**
	 * {@inheritDoc}
	 */
	public Value createValue(Node value) throws IllegalValueException {
		return new ReferenceValue(value);
	}

	/**
	 * {@inheritDoc}
	 */
	public Value createValue(String value) {
		return new StringValue(value);
	}

	/**
	 * {@inheritDoc}
	 */

	public static Value createValue(String value, String type) throws IllegalValueException {
		int itype = PropertyType.STRING;
		try {
			itype = Integer.parseInt(type);
		} catch (NumberFormatException e) {
			// e.printStackTrace();
			itype = PropertyType.valueFromName(type);
		}

		return createValue(value, itype);
	}

	public static Value createValue(String value, int type) throws IllegalValueException {
		Value val;
		switch (type) {
		case PropertyType.STRING:
			val = new StringValue(value);
			break;
		case PropertyType.BOOLEAN:
			val = BooleanValue.valueOf(value);
			break;
		case PropertyType.DOUBLE:
			val = DoubleValue.valueOf(value);
			break;
		case PropertyType.FLOAT:
			val = FloatValue.valueOf(value);
			break;
		case PropertyType.LONG:
			val = LongValue.valueOf(value);
			break;
		case PropertyType.DATE:
			try {
				val = DateValue.valueOf(value);
			} catch (IllegalValueException e) {
				try {
					long longtime = Long.valueOf(value);
					Calendar cal = Calendar.getInstance();
					cal.setTimeInMillis(longtime);
					val = new DateValue(cal);
					break;
				} catch (NumberFormatException ex) {
					throw new IllegalValueException(ex.getMessage());
				}

			}
			break;

		case PropertyType.NAME:
			val = NameValue.valueOf(value);
			break;
		case PropertyType.PATH:
			val = PathValue.valueOf(value);
			break;
		case PropertyType.REFERENCE:
			val = ReferenceValue.valueOf(value);
			break;
		case PropertyType.HTTPLINK:
			val = HTTPLinkValue.valueOf(value);
			break;
		case PropertyType.INTEGER:
			val = IntegerValue.valueOf(value);
			break;
		case PropertyType.BINARY:
			val = new BinaryValue(value);
			break;
		case PropertyType.ARRAYLIST_STRING:
			val = new ArrayListValue(value);
			break;
		case PropertyType.SET_STRING:
			val = new StringSetValue(value);
			break;
		default:
			throw new IllegalArgumentException("Invalid type constant: " + type);
		}
		return val;
	}

	public static Value genValue(Object node, String fieldName) {

		try {
			Class c = node.getClass();
			Field field = null;
			//System.out.println(c.toString());
			try {
				field = c.getDeclaredField(fieldName);
			}catch (NoSuchFieldException e) {
				field = c.getField(fieldName);
			}
			field.setAccessible(true);
			return genValue(node, field);
		} catch (SecurityException e) {
			e.printStackTrace();
			return null;
		}catch (Exception e) {
			e.printStackTrace();
			return null;
		} 

	}

	public static Value genValue(Object node, Field field) {
		if (field == null || node == null) {
			return null;
		}
		Value value = null;
		try {
			String fieldtype = field.getType().toString();
			int type = PropertyType.valueTypeFromJavaClass(fieldtype);
			try {
				switch (type) {
				case PropertyType.INTEGER:
					value = new IntegerValue(((Integer) field.get(node)));
					break;
				case PropertyType.STRING:
					value = new StringValue((String) field.get(node));
					break;
				case PropertyType.DATE:
					value = new DateValue((Calendar) field.get(node));
					break;
				case PropertyType.DATETIME:
					Date date = (Date) field.get(node);
					Calendar cal = Calendar.getInstance();
					cal.setTime(date);
					value = new DateValue(cal);
					// value = new DateValue((Calendar) field.get(node));
					break;
				case PropertyType.BOOLEAN:
					value = new BooleanValue((Boolean) field.get(node));
					break;
				case PropertyType.DOUBLE:
					Object obj = field.get(node);
					if(obj instanceof Double){
						value = new DoubleValue((Double)obj);
					}else if(obj instanceof Float){
						value = new DoubleValue((Float)obj);
					}
					
					break;
				case PropertyType.FLOAT:
					obj = field.get(node);
					if(obj instanceof Double){
						value = new FloatValue((Double)obj);
					}else if(obj instanceof Float){
						value = new FloatValue((Float)obj);
					}
					
					break;
				case PropertyType.ARRAYLIST_STRING:
					value = new ArrayListValue((List<String>) field.get(node));
					break;
				case PropertyType.SET_STRING:
					value = new StringSetValue((Set<String>) field.get(node));
					break;

				}
			} catch (Exception e) {
				e.printStackTrace();
				return value;
			}
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
		return value;
	}

	public static boolean setField(Object node, Field field, Value value) {
		field.setAccessible(true);
		String fieldtype = field.getType().toString();
		int type = PropertyType.valueTypeFromJavaClass(fieldtype);
		try {
			switch (type) {
			case PropertyType.INTEGER:
				field.set(node, value.getInt());
				break;
			case PropertyType.STRING:
				field.set(node, value.getString());
				break;
			case PropertyType.DATE:
				field.set(node, value.getDate());
				break;
			case PropertyType.DATETIME:
				field.set(node, value.getDate().getTime());
				break;
			case PropertyType.BOOLEAN:
				field.set(node, value.getBoolean());
				break;
			case PropertyType.DOUBLE:
				field.set(node, value.getDouble());
				break;
			case PropertyType.FLOAT:
				field.set(node, (float)value.getDouble());
				break;
			case PropertyType.ARRAYLIST_STRING:
				if (value instanceof ArrayListValue) {
					ArrayListValue v = (ArrayListValue) value;
					field.set(node, v.get_data());
				}
				break;
			case PropertyType.SET_STRING:
				if (value instanceof StringSetValue) {
					StringSetValue v = (StringSetValue) value;
					field.set(node, v.get_data());
				}
				break;
			}

		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}

	public static Object setFieldViaSetter(Object node, Field field, Value value) throws NoSuchMethodException {
		return setFieldViaSetter(node, field.getName(), field.getType().toString(), value);
	}

	public static Object setFieldViaSetter(Object node, String fieldName, String fieldtype, Value value) throws NoSuchMethodException {
		Class c = node.getClass();
		int type = PropertyType.valueTypeFromJavaClass(fieldtype);
		try {
			switch (type) {
			case PropertyType.INTEGER: {
				int data = 0;
				try {
					data = value.getInt();
				} catch (Exception e) {
					e.printStackTrace();
					return false;
				}
				Class[] parameterTypes = { int.class };
				Object[] arguments = new Object[] { data };
				Method method = c.getMethod("set" + fieldName, parameterTypes);
				method.invoke(node, arguments);
				break;
			}
			case PropertyType.STRING: {

				// field.set(node,str);
				String data = null;
				// data=Integer.parseInt(str);
				Class[] parameterTypes = { String.class };
				try {
					data = value.getString();
				} catch (Exception e) {
					// e.printStackTrace();
				}
				Object[] arguments = new Object[] { data };
				Method method = c.getMethod("set" + fieldName, parameterTypes);
				method.invoke(node, arguments);

				break;

			}
			case PropertyType.DATE: {

				Calendar cal = null;
				try {
					cal = value.getDate();
				} catch (Exception e) {
					cal = CalendarFormat.NULL_CALENDAR;
				}
				Class[] parameterTypes = { Calendar.class };
				Object[] arguments = new Object[] { cal };
				Method method = c.getMethod("set" + fieldName, parameterTypes);
				method.invoke(node, arguments);

				break;

			}
			case PropertyType.DATETIME: {

				Calendar cal = null;
				try {
					cal = value.getDate();
				} catch (Exception e) {
					cal = CalendarFormat.NULL_CALENDAR;
				}
				Class[] parameterTypes = { Date.class };
				Object[] arguments = new Object[] { cal.getTime() };
				Method method = c.getMethod("set" + fieldName, parameterTypes);
				method.invoke(node, arguments);

				break;

			}
			case PropertyType.BOOLEAN: {
				boolean data = false;
				try {
					data = value.getBoolean();
				} catch (Exception e) {
					e.printStackTrace();
					return false;
				}
				Class[] parameterTypes = { boolean.class };
				Object[] arguments = new Object[] { data };
				Method method = c.getMethod("set" + fieldName, parameterTypes);
				method.invoke(node, arguments);

				break;

			}
			case PropertyType.DOUBLE: {
				double data = 0.0d;
				try {
					data = value.getDouble();
				} catch (Exception e) {
					e.printStackTrace();
					return false;
				}

				// field.set(node,d.doubleValue());
				Class[] parameterTypes = { double.class };
				Object[] arguments = new Object[] { data };
				Method method = c.getMethod("set" + fieldName, parameterTypes);
				method.invoke(node, arguments);

				break;

			}
			case PropertyType.FLOAT: {
				float data = 0.0f;
				try {
					data = new Float(value.getDouble());
				} catch (Exception e) {
					e.printStackTrace();
					return false;
				}

				// field.set(node,d.doubleValue());
				Class[] parameterTypes = { float.class };
				Object[] arguments = new Object[] { data };
				Method method = c.getMethod("set" + fieldName, parameterTypes);
				method.invoke(node, arguments);

				break;

			}
			case PropertyType.ARRAYLIST_STRING: {

				if (value instanceof ArrayListValue) {
					ArrayListValue listValue = (ArrayListValue) value;
					List<String> list = listValue.get_data();
					Class[] parameterTypes = { List.class };
					Object[] arguments = new Object[] { list };
					Method method = c.getMethod("set" + fieldName, parameterTypes);
					method.invoke(node, arguments);
					break;
				}
				return false;

			}
			case PropertyType.SET_STRING: {

				if (value instanceof StringSetValue) {
					StringSetValue setValue = (StringSetValue) value;
					Set<String> set = setValue.get_data();
					Class[] parameterTypes = { Set.class };
					Object[] arguments = new Object[] { set };
					Method method = c.getMethod("set" + fieldName, parameterTypes);
					method.invoke(node, arguments);
					break;
				}
				return false;

			}
			}
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
			return false;
		} catch (IllegalAccessException e) {
			e.printStackTrace();
			return false;
		} catch (InvocationTargetException e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}

	public static boolean setFieldFromStringViaSetter(Object node, Field field, String str) throws NoSuchMethodException {
		return setFieldFromStringViaSetter(node, field.getName(), field.getType().toString(), str);
	}

	public static boolean setFieldFromStringViaSetter(Object node, String fieldName, String fieldtype, String str) throws NoSuchMethodException {
		Class c = node.getClass();
		int type = PropertyType.valueTypeFromJavaClass(fieldtype);
		try {
			switch (type) {
			case PropertyType.INTEGER: {
				int data = 0;
				data = Integer.parseInt(str);
				Class[] parameterTypes = { int.class };
				Object[] arguments = new Object[] { data };
				Method method = c.getMethod("set" + fieldName, parameterTypes);

				method.invoke(node, arguments);

				break;
			}
			case PropertyType.STRING: {

				Class[] parameterTypes = { String.class };
				Object[] arguments = new Object[] { str };
				Method method = c.getMethod("set" + fieldName, parameterTypes);
				method.invoke(node, arguments);
				break;

			}
			case PropertyType.DATETIME:
			case PropertyType.DATE: {

				str = str.trim();
				if (str.length() <= 10) {
					Calendar cal = ISO8601.parse(str);
					if (cal == null) {
						cal = CalendarFormat.getCalendar4(str);
					}
					Class[] parameterTypes = { Calendar.class };
					Object[] arguments = new Object[] { cal };
					Method method = c.getMethod("set" + fieldName, parameterTypes);
					method.invoke(node, arguments);
					break;
				}

			}
			case PropertyType.BOOLEAN: {

				boolean b = Boolean.valueOf(str);
				// field.set(node,b);
				Class[] parameterTypes = { boolean.class };
				Object[] arguments = new Object[] { b };
				Method method = c.getMethod("set" + fieldName, parameterTypes);
				method.invoke(node, arguments);

				break;

			}
			case PropertyType.DOUBLE: {

				Double d = Double.parseDouble(str);
				double data = d.doubleValue();
				// field.set(node,d.doubleValue());
				Class[] parameterTypes = { double.class };
				Object[] arguments = new Object[] { data };
				Method method = c.getMethod("set" + fieldName, parameterTypes);
				method.invoke(node, arguments);

				break;

			}
			case PropertyType.FLOAT: {

				Float d = Float.parseFloat(str);
				double data = d.doubleValue();
				// field.set(node,d.doubleValue());
				Class[] parameterTypes = { float.class };
				Object[] arguments = new Object[] { data };
				Method method = c.getMethod("set" + fieldName, parameterTypes);
				method.invoke(node, arguments);

				break;

			}
			case PropertyType.ARRAYLIST_STRING: {

				// System.out.println("PropertyType.ARRAYLIST_STRING:");
				Class[] parameterTypes = { List.class };
				List<String> list = ArrayListValue.strToList(str, null);
				Object[] arguments = new Object[] { list };
				Method method = c.getMethod("set" + fieldName, parameterTypes);
				method.invoke(node, arguments);
				break;

			}
			case PropertyType.SET_STRING: {

				// System.out.println("PropertyType.ARRAYLIST_STRING:");
				Class[] parameterTypes = { Set.class };
				Set<String> list = StringSetValue.strToSet(str, null);
				Object[] arguments = new Object[] { list };
				Method method = c.getMethod("set" + fieldName, parameterTypes);
				method.invoke(node, arguments);
				break;

			}
			}
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
			return false;
		} catch (IllegalAccessException e) {
			e.printStackTrace();
			return false;
		} catch (InvocationTargetException e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}

	public static Value cloneHelper(Value value) {
		if (value == null) return null;
		int type = value.getType();

		// System.out.println(valueStr);

		String valueStr = null;
		Value newValue = null;
		try {
			valueStr = value.getString();
			newValue = createValue(valueStr, type);
		} catch (IllegalValueException e) {
			e.printStackTrace();
			return null;
		} catch (GLException e) {
			e.printStackTrace();
			return null;
		}
		return newValue;
	}

}